<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC2/class/dao/extended/help.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: help.php 4533 2006-06-26 23:52:30Z wenzel $

/**
  * help.php - contains the {@link Help} class
  *
  * @author $Author: wenzel $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

require_once DAODIR.'/Help.php';

/**
  * The class for the built-in help files
  *
  * @package    classes
  * @subpackage dao
  */
class Help extends DAO_Help {

    // {{{ constructor

    /**
      * Constructor
      *
      * @param  string  name identifier of this help objects
      */
    function Help ( $help_ident =0 ) 
    {
        if ($help_ident) {
            $this->ident = $help_ident;
            $this->get('ident', $this->ident);
        }

    } // Help 

    // }}}

/* Setters */
    // {{{ setContent()
    function setContent($content)
    {
        if (!$content)
            $content = $this->content;

        $this->content = stripslashes($content);
        return true;
    }

    // }}}

    // {{{ title()

    /**
      * Return the content as a valid HTML link TITLE text
      *
      * @return string  HTML content
      */
    function title() 
    {
        if (!substr_count($_SERVER['HTTP_USER_AGENT'], "MSIE")) {
            $this->content = preg_replace('/\\n|\\r|\\t/',' ',$this->content);
        }

        return $this->content;

    } // title

    // }}}
    // {{{ display_help_select() (deprecated)
    /**
      * Display all help topics as <option> </option>
      *
      * Note: This really belongs in UI
      *
      * @deprecated new one will return a Container
      * @return string  HTML list of OPTION tags
      */
    function display_help_select()
    {

        // "SELECT ident FROM help ORDER BY ident ASC"
        $dbdo = DB_DataObject::Factory('help');
        $dbdo->selectAdd();
        $dbdo->selectAdd('ident');
        $dbdo->orderBy('ident ASC');
        $dbdo->find();

        $help = '';
        while ($dbdo->fetch())
            $help .= "<option value=\"$dbdo->ident\">$dbdo->ident</option>\n";

        return $help;            

    } // display_help_select

    // }}}
    // {{{ show_link()

    /**
      * Generate a link for this help object
      *
      * @param  string  text for the link
      * @return object  phpHtmlLib container representing the link
      */
    function show_link($help_link_text='') 
    {
        global $conf;

        $link   = web_root() . "help.php?ident=$this->ident";
        $url    = "javascript:Start('$url')";

        if ($this->content) {
            $content = '[?]';
        } else {
            $content = '[N]';
        }

        if ($help_link_text) {
            $content = $help_link_text;
        }

        return html_a($url, $content);

    } // show_link

    // }}}
    // {{{ show_admin_link()

    /**
      * Generate an 'edit' link for this help object
      *
      * @param  string  link text
      * @return object  phpHtmlLib container
      */
    function show_admin_link($help_link_text='')
    {
        global $conf;

        $link   = web_root() . "help.php?action=edit&amp;ident=$this->ident";
        $url    = "javascript:Start('$url')";

        if ($this->content) {
            $content = '[E]';
        } else {
            $content = '[N]';
        }

        $first  = $this->show_link();
        $next   = html_a($url, $content);

        return container($first, $next);
    }

    // }}}

} // class Help

?>
