<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC2/class/dao/extended/user.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// |            Zack Bartel <bartel@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: user.php 4763 2006-08-21 18:13:19Z wenzel $

/**
  * user.php - contains the {@link User} class
  *
  * @author $Author: wenzel $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

/*!
	@header User Class
	This is the base user class which will be stored in the session
	information for each connecting user.

*/

define("LOGIN", true);

include_once DAODIR.'/User.php';
require_once INCLUDEDIR.'/auth.php';

/**
  * This class represents a user in Maintain
  *
  * @package    classes
  * @subpackage dao
  */
class User extends DAO_User {

    // {{{ vars

    /** from zone_user */
    var $level;

    /** map 'level's value to a zone (for zone_user) */
	var $current_zone;

    // }}}

    // {{{ User (constructor)

    function User ($uname = '', $login = false) {
        global $conf;

        if ($uname) {

            $this->username = $uname;
            $success = $this->get($this->username);

            // if we're logging in, update last_seen
            if ($login) {
                if ($success) {
                    $this->last_seen = time();
                    $this->update();
                }
            } //if login

        }
    }

    // }}}

    /* DB_DataObject Overloads */
    // {{{ update()

    /**
      * Update stored information for User and Zone_user in the DB
      *
      * Note:  If creating zone/level mappings, run this function, and NOT insert!
      *
      * @param  db_dataobject   base object for only updating differences
      * @return mixed           number of updated rows, or false on failure
      */
    function update($dbdo = false) {
        global $conf;

        // before we update/insert, we need to make sure the password
        // is going to be hashed properly if needed
        $this->_update_password();

        $rows = parent::update($dbdo);

        if (parent::update($dbdo) === false)
            return false;

        // since we're feigning multiple inheritance by implementing
        // both User and Zone_user in this class, we have to update
        // this too

        // we only need to do this if an access level has been set
        if ($this->level) {

            // let's check if this mapping exists
            $dbdo = DB_DataObject::Factory('Zone_user');
            $dbdo->username = $this->username;
            $dbdo->zone     = $this->current_zone;

            // if mapping exists, update it
            if ($dbdo->find(true)) {
                $dbdo->level    = $this->level;
                $rows2 = $dbdo->update();
            }

            // otherwise, insert it
            else {
                $dbdo->level    = $this->level;
                $rows2 = $dbdo->insert();

                if ($rows2 === false)
                    return false;
            }


            // only return whichever one had modified rows
            // (or more rows I guess...assuming 0 vs 1)
            if ($rows && $rows2 > $rows)
                $rows = $rows2;

        }

        // (re)build objects
        $this->setAll();

        return $rows;

    }
    // }}}
    // {{{ insert()
    function insert()
    {
        // before we update/insert, we need to make sure the password
        // is going to be hashed properly if needed
        $this->_update_password();

        return parent::insert();
    }

    // }}}
    // {{{ delete()
    
    /**
      * Delete user and zone_user mappings
      *
      * @param  boolean $useWhere   true to use whereAdd()s, not just primary key(s)
      * @return mixed   number      of users deleted, or false on failure
      */
    function delete($useWhere = false) {
        
        global $user;
        /* Make sure this is a valid user object */ 
        if (!isset($this->username))    {
            return false;
        }

        /* No deleting self */        
        if (!strcasecmp($user->username, $this->username))   {
            return false;
        }
        /* Delete all zone-user mappings first */
         $zu = DB_DataObject::Factory("zone_user");
         $zu->username = $this->getUsername();
         $zu->find();
         
         while($zu->fetch()) {
             if(!$zu->delete()) {
                $res = false;
             } /*else {
                 $user->write_history(_("User Deleted"), ACTION_DELETE, 
                                     $user->username, $_SESSION['zone']);
             }*/
         }
        /* If something already failed let me know */
        if ($res === false)
            return $res;

        /* Now delete the user */

        $res = parent::delete($useWhere);
        return $res;
    }

    // }}}
    // {{{ setFrom()

    /**
      * extended from DB_DataObject::setFrom() to include Zone_user vars
      */
    function setFrom($from, $format = '%s') {

        $val = parent::setFrom($from, $format);

        if (is_object($from))
            $this->setLevel($from->level);
        elseif (is_array($from))
            $this->setLevel($from['level']);
        else
            return false;

        return $val;
    }
 
    // }}}

    /* Setters */
    // {{{ setAll()
  
    /**
      * Set all internal variabes/objects
      * to default values, or re-initialize
      * to current values
      */
    function setAll() {
        global $conf;

        $this->setEmail($this->email);
        $this->setColor_template($this->color_template);
        $this->setHost_view($this->host_view);
        $this->setUi_size($this->ui_size);
        $this->setPreferred_zone($this->preferred_zone);
        $this->setList_view($this->list_view);
        $this->setLast_seen($this->last_seen);

    }

    // }}}
    // {{{ setName()
    function setUsername($name = '')
    {
        $this->username = $name;
        $this->setEmail($this->email);
    }
    // }}}
    // {{{ setEmail()

    /**
      * Set the user's e-mail address
      *
      * @return boolean True
      */
    function setEmail($mail = '') {
        global $conf;

        if (!$mail) {
            if ($this->email)
                $mail = $this->email;
            else
                $mail = $this->username . '@' . $conf['info']['email_domain'];
        }
        else
            $mail = stripslashes($mail);

        $this->email = $mail;

        return true;

    }
    
    // }}}
    // {{{ setZone()

    /**
      * Set a new current_zone and access level for user object
      * 
      * @param  int     $zone_id    new zone ID
      * @param  boolean $setlevel   true if function should set associated user access level as well
      * @return boolean true for DBDO inheritance
      */
    function setZone($zone_id, $setlevel = true) 
    {
        global $conf;

        // if there is a zone set, use it, otherwise use preferred
        // this also sets the user's access level
        if (!$zone_id) {

            // if one's already set, leave it
            if ($this->current_zone)
                $zone_id = $this->current_zone;

            // next, try current zone
            elseif ($_SESSION['zone'])
                $zone_id = $_SESSION['zone'];

            // or try user's preferred zone
            elseif ($this->preferred_zone && $this->get_access_level($this->preferred_zone))
                $zone_id = $this->preferred_zone;

            // or take the first (lexical) zone user's in
            else {
                $zones = $this->zones();
                $zone_id = $zones[0];
            }

        } // if no zone given

        // update the zone
        $this->current_zone = $zone_id;

        // if the user wants to pull down the associated level as well, do it
        if ($setlevel)
            $this->setLevel();

        return true;
    }

    // }}}
    // {{{ setLevel()

    /**
      * Set the access level for the corresponding zone
      *
      * if called with no arguments:
      *     level is read from the database
      *     zone is current_zone
      *
      * @param  int     access level
      * @param  int     zone ID associated with access level
      * @return boolean true on success, false on failure
      */
    function setLevel($level = false, $zone = false) {
        global $conf;

        // cannot set a user to ADMIN like this
        if ($level AND $level == ADMIN)
            return false;

        // make sure we're dealing with the right zone
        if ($zone && $zone != $this->current_zone)
            $this->setZone($zone, false);

        // make sure a zone is set
        if (!isset($this->current_zone))
            return false;

        // ok, if no level was set, read it in
        if (!$level) {

            // ok, kinda ugly, but to avoid excessive DB hits for is_admin(),
            // if we've determined them to be ADMIN already, leave them there
            if ($this->level == ADMIN)
                $level = ADMIN;

            else {
                $dbdo = DB_DataObject::Factory('zone_user');
                $dbdo->username = $this->username;
                $dbdo->zone     = $zone;

                // if no mapping exists, make sure no level is set
                if (!$dbdo->find(true)) {
                    $level = null;
                    return false;
                }

                $level = $dbdo->level;
            }
        } // if no level given

        // if no problems, set the new level
        $this->level = $level;
        return true;

    }

    // }}}
    // {{{ setColor_template()
    function setColor_template($template) 
    {
        global $conf;

        if (!$template) {
            $this->color_template = $conf['general']['default_skin'];
        } else {
            $this->color_template = $template;
        }

        return true;
    }

    // }}}
    // {{{ setHost_view()

    /**
      * Set the complexity of the UI
      *
      * @param  string  'basic' or 'advanced'
      * @return boolean true on success
      */
    function setHost_view($view) {
   
        /* Order of attempts:
         *  1) argument
         *  2) internal variable
         *  3) default to 'basic'
         */
        if (!$view)
            $view = ($this->host_view) ? $this->host_view : 'basic';

        $this->host_view = $view;

        return true;
    }

    // }}}
    // {{{ setType()

    /**
      * Set the authentication method
      *
      * Current options are 'ldap' or 'auth_maintain'
      *
      * @param  string  authentication method
      * @return boolean true on success
      */
    function setType($type) 
    {
        global $conf;

        // set the type
        $this->type = strtolower($type);

        return true;

    }

    // }}}
    // {{{ setPassword()

    /**
      * Set the user's password
      *
      * @param  string  password
      * @return boolean true on success
      */
    function setPassword($passwd = '') 
    {
        $this->password = $passwd;
        return true;
    }

    // }}}
    // {{{ setUi_size()

    /**
      * Set a default ui size if none specified in the DB
      *
      * @param  int     $size   font size (in px)
      * @return boolean true on success
      */
    function setUi_size($size)
    {
        global $conf;

        // if no size specified, set default
        if (!$size)
            $size = ($this->ui_size) ? $this->ui_size : $conf['general']['size'];

        $this->ui_size = $size;

        return true;

    }
    //}}}
    // {{{ setLast_seen()
    function setLast_seen($ls)
    {
        if (!$ls) {
            $this->last_seen = 0;
        } else {
            $this->last_seen = $ls;
        }

        return true;
    }
    // }}}
    // {{{ setPreferred_zone()
    function setPreferred_zone($zn)
    {
        // fall back to global $_SESSION['zone']
        if (!$zn) {
            // if no global zone, we're game over
            if (!$_SESSION['zone']) {
                return false;
            }
            $this->preferred_zone = $_SESSION['zone'];
        } else {
            $this->preferred_zone = $zn;
        }

        return true;
    }
    // }}}
    // {{{ setList_view()
    function setList_view($lv)
    {
        global $conf;

        if (!$lv) {
            $this->list_view = $conf['general']['max_list'];
        } else {
            $this->list_view = $lv;
        }

        return true;
    }
    // }}}

    /* Validators */
    // {{{ validateEmail()

    /**
      * Validate specified e-mail address
      *
      * @return boolean False if e-mail invalid
      */
    function validateEmail() 
    {
        global $conf;

        if ($this->getEmail() == '')
            return true;
        // if admin requires username@email_domain :
        if ($conf['info']['force_email_domain'] === true) {

            // this forces username@email_domain addresses
            $valid_email = $this->username.'@'.$conf['info']['email_domain'];

            if (strcmp($valid_email, $this->email)) 
                return _("Email must remain username").'@'.$conf['info']['email_domain'];
        }

        // otherwise, just make sure it's syntactically valid
        /*
        else
            return Validate::email($this->email, false);
        */

        /* having looked at Validate's email function
         * I've realized it doesn't work as a regex at all!
         * so here's one I found on regular-expressions.info
         */
        else {
            $valid_email = preg_match("/^[A-Z0-9._%-]+@[A-Z0-9._%-]+\.[A-Z]{2,4}$/i", $this->email);
            if (!$valid_email)
                return _("You have entered an invalid email address");
        }

        return true;
    }

    // }}}
    // {{{ validateLevel()

    /**
      * Make sure a valid level is set
      *
      * @return boolean true if level is a defined value
      */
    function validateLevel() 
    {

        $vals = array(ADMIN, ZONE_ADMIN, ZONE_USER, GUEST);

        if (array_search($this->level, $vals) !== false)
            return true;

        else {
            $msg = _("Invalid level. Valid levels are:").'  ';
            $msg .= implode(',', $vals);
            return $msg;
        }

        return true;
    }
    
    // }}}
    // {{{ validatePassword()

    /**
      * Make sure the password is valid
      *
      * @return boolean true if valid password
      */
    function validatePassword() 
    {
        global $conf, $user;

        // way too many hard-coded validation types

        //  if this is not a mysql user, there's nothing to validate
        if ($this->type != DEFAULT_AUTH)
            return true;

        // make sure username is set
        if (!$this->username)
            return _("Attempt to set password for null username unsuccessful");

        // make sure the password is long enough
        if (strlen($this->password)<PASSWORD_MIN_LEN)
            return _("Password must be at least").' '.PASSWORD_MIN_LEN.' '._("characters");

        // make sure password isn't all letters, all digits, and doesn't contain original passwd
        if (preg_match("/^[A-Za-z]+$|^[0-9]+$|changeme/", $this->password))
            return _("New password must not be all letters, all numbers, or contain your previous password");

        // if we got this far, hash the password and remove the "new" flag

        return true;
    }

    // }}}
    // {{{ validateType()
    
    function validateType() 
    {
        global $conf;

        // make sure a type is set
        if (!$this->type) {
            return _("No authentication type was set for this user");
        }

        // if this type is not enabled, don't allow it
        if (!is_auth_enabled($this->type)) {
            return _("Invalid authentication type specified");
        }

        return true;

    }
    // }}}
    // {{{ validateUi_size()
    
    /**
      * Ensure font size remains properly set
      *
      * @return boolean true if integer between 2 and 99
      */
    function validateUi_size() 
    {
        global $conf;

        // hurray for more hard-coded values
        $valid = (2<=$this->ui_size AND $this->ui_size<=99);

        if (!$valid)
            return _("Valid font sizes are between 2 and 99");

        return true;
    }
    // }}}
    // {{{ validateUsername()

    /**
      * Validates username string, and checks permissions/existance
      *
      * @return boolean false on failure
      */
    function validateUsername() 
    {
        global $conf;

        // make sure a name was set
        if (!$this->username)
            return _("You must specify a username for new user");

        // make sure it's just characters
        if (!ereg("^[[:alpha:]]+$", $this->username))
            return _("Username must consist only of letters");

        // if we're not in the database, make sure this user exists
        // on the authentication server
        // (note: not needed for maintain_auth as they wouldn't exist
        // yet if we were creating a new user)
        if (!$this->type || $this->type == DEFAULT_AUTH) {
            $auth_type  = User::check_auth_name($this->username);

            // if user doesn't exist, error out 
            if (PEAR::isError($auth_type)) {
                return $auth_type->getMessage();
            }

        }

        // if we made it here, we're good to go
        return true;

    }

    // }}}
//{{{ validatePreferred_zone()

    function validatePreferred_zone()
    {
        return true;
    }
//}}}
//{{{   getPassword()
    /*  Gets the user's password
     *
     * @return String password (sha1)
     */
    function getPassword()
    {
        return $this->password;
    }
//}}}

    /* User Functions */
    // {{{ get_level()

	/**
      * get the access level for this user
      * based on a zone
      *
      * @return int user access value (ADMIN, ZONE_GUEST, etc.)
      */
	function get_level($zone_id = false) 
    {
		global $conf, $user;

		if (!$zone_id)
            $zone_id = $_SESSION['zone'];

        // if the user is already set for this zone,
        // return the stored values
        if ($this->current_zone == $zone_id && isset($this->level))
            return $this->level;

        // "SELECT level FROM zone_user WHERE username='$this->username' AND zone='$zone'"
        $dbdo = DB_DataObject::Factory('zone_user');
        $dbdo->selectAdd();
        $dbdo->selectAdd('level');
        $dbdo->username = $this->username;
        $dbdo->zone = $zone_id;
        if (!$dbdo->find(true))
            return NO_ACCESS;

        return $dbdo->level;

	} // get_level

    // }}}
    // {{{ get_last_updated_object()

	//get the last object that they updated 
	function get_last_updated_object() 
    {
		global $conf;

        // "SELECT * FROM object_history WHERE username='$this->username' ORDER BY date DESC"
        $dbdo = DB_DataObject::Factory('object_history');
        $dbdo->username = $this->username;
        $dbdo->orderBy('date DESC');
        $results = $dbdo->find(true);

        $link = '';

        /* Now do work depending on what type of object it is */
        switch ($results->object_type) {

        case 'host':
            $results->object = new Host($results->object_id);
            $results->name = $results->object->hostname;
            $link = "host.php?action=show&amp;id=";
            break;

        case 'domain':
            $results->object = new Domain($results->object_id);
            $results->name = $results->object->name;
            $link = "domain.php?action=show&amp;id=";
            break;

        case 'subnet':
            $results->object = new Subnet($results->object_id);
            $results->name = $results->object->name;
            $link = "domain.php?action=show&amp;id=";
            break;

        case 'cname':
            $results->object = new Cname($results->object_id);
            $results->name = $results->object->hostname;
            $link = "cname.php?action=show&amp;id=";
            break;

        case 'mx':
            $results->object = new Mx($results->object_id);
            $results->name = $results->object->hostname;
            $link = "mx.php?action=show&amp;id=";
            break;

        case 'zone':
            $results->object = new Zone($results->object_id);
            $results->name = $results->object->name;
            $link = "zone.php?action=show&amp;id=";
            break;

        case 'pointer':
            $results->object = new Pointer($results->object_id);
            $results->name = $results->object->type . " pointer (" .$results->object->hostname . " <--> " . IP::int2ip($results->object->ip) . ")";
            $link = "pointer.php?action=show_pointer&amp;pointer_id=";
            break;

        default:
            $results->object = new stdClass;
            $results->name = "Invalid Object Type";
            $results->link = "index.php";
            return $results;

        } //switch

        // finish building the link
        $results->link = web_root() . $link . $results->object_id;

    	return $results;

	} //get_last_updated_object

    // }}}
    // {{{ zones()

    /**
      * Get all zones given user has permissions for
      *
      * @param  int     only return zones where user has >= this access level
      * @return array   array of zone IDs sorted by zone name
      */
	function zones ($level = 0) 
    {
        $zones = array();
        
        if ($this->is_admin())    {
            $dbdo = new Zone();
            $dbdo->selectAdd();
            $dbdo->selectAdd('id');

            $dbdo->orderBy('name');    
    
            $dbdo->find();

            while ($dbdo->fetch())
                $zones[] = $dbdo->getId();
        }
        else    {
            $dbdo = DB_DataObject::Factory('zone_user');
            $dbdo->setUsername($this->getUsername());

            $dbdo->selectAdd();
            $dbdo->selectAdd('zone');  
        
            $dbdo->find();
    
            while ($dbdo->fetch())
                $zones[] = $dbdo->getZone();
        }

		return $zones;
	} // zones

    // }}}
    // {{{ access_level()

    /**
      * Translates access level integer into
      * string description
      *
      * can be called statically
      */
	function access_level($level=false) 
    {
		global $conf;

        if (!$level)
            $level = $this->get_level();

        switch ($level) {

        case ADMIN:
            $results = "Full Admin";
            break;

        case ZONE_ADMIN:
            $results = "Zone Admin";
            break;

        case ZONE_USER:
            $results = "Zone User";
            break;

        case GUEST:
            $results = "Zone Guest";
            break;

        default:
            $results = "Undefined User Level";
            break;
        }

	    return $results;

	} //access_level

    // }}}
    // {{{ is_setup_user()

    /**
      * Checks to see if this is the default "admin" account
      *
      * @return boolean
      */
	function is_setup_user() 
    {
		global $conf;

		if ($this->username === "admin") { 
            // "SELECT username FROM user WHERE username='admin' AND password=password('changeme')"
            $dbdo = DB_DataObject::Factory('user');
            $dbdo->username = 'admin';
            $dbdo->whereAdd("password='".sha1('changeme')."'");
            if ($dbdo->find() > 0)
                return true;
        }

		return false;

	} // is_setup_user

    // }}}
    // {{{ add_full_admin()

    /**
      * Adds a full admin user
      *
      * @param  string  $username   name of user to add
      * @return boolean true on success or user is already an admin, false on failure
      */
	function add_full_admin($username) 
    {
		global $conf;

        if (User::is_admin($username))
            return true;
        else {
            // "INSERT INTO zone_user SET username='$username', level='100', zone='0'"
            $dbdo = DB_DataObject::Factory('zone_user');
            $dbdo->username = $username;
            $dbdo->level = ADMIN;
            $dbdo->zone = 0;
            if ($dbdo->insert())
                return true;
		}

        return false;

	} // add_full_admin

    // }}}
    // {{{ check_auth_name()

    /**
      * Verify that given username exists on some authentication server
      *
      * Each enabled authentication module will be queried until
      * a match is found or all servers exhausted and no match found
      *
      * @static
      * @param  string  username
      * @param  mixed   string or array of auth modules to query
      * @return mixed   name of module capable of authenticating this user
      *                 or false if user doesn't exist
      */
    function check_auth_name($name, $type = '')
    {
        global $conf;

        // if no type was specified, try each type in the config
        if (!$type) {
            $type = valid_auth_types();

        // cast the type to an array for the foreach loop
        } elseif (is_string($type)) {
            $type = array($type);

        }

        // iterate through each auth type until a match is found
        foreach ($type as $auth_type) {

            $mod = new $auth_type;

            if ($mod->hasUser($name)) {
                return $mod->name;
            }
        }

        // if we're here, the user didn't exist
        return false;

   } //check_auth_name

    // }}}

    /* User Level Checkers */
    // {{{ get_access_level()

    /**
      * Return access level user has for given zone
      *
      * @param  int     zone ID
      * @return int     user's access level
      */
	function get_access_level($zone) {
		global $conf;

        if (!$zone)
            $zone = $this->current_zone;

        // if they're an admin, return as much
        if ($this->is_admin())
            return ADMIN;

        $dbdo = DB_DataObject::Factory('zone_user');
        $dbdo->selectAdd();
        $dbdo->selectAdd('level');
        $dbdo->zone     = $zone;
        $dbdo->username = $this->username;

        // if no mappings found, no access
        if (!$dbdo->find(true))
            return NO_ACCESS;
        else
            return $dbdo->level;

	} //get_access_level

    // }}}
    // {{{ is_admin()

    /**
      * Test if user has full admin rights
      *
      * @return boolean true if user has full ADMIN rights
      */
	function is_admin($usr = false) {
		global $conf;

        if ($this->level == ADMIN)
            return true;

        // if you're an admin, you're an admin...zone doesn't matter
        $dbdo = DB_DataObject::Factory('zone_user');
        $dbdo->username = $this->username;
        $dbdo->whereAdd("level=".ADMIN);

        if ($dbdo->find())
            return true;
        else
            return false;

	} //is_admin

    // }}}
    // {{{ is_zone_admin()

    /**
      * Test if user is a zone admin or up
      *
      * @param  int     zone id to check against
      * @return boolean true if user has zone admin rights or above
      */
	function is_zone_admin($zone = false)  {

        // zone admin is a bit tricky: admin counts as zone_admin
        if ($this->get_access_level($zone) >= ZONE_ADMIN)
            return true;

        return false;

	} //is_zone_admin

    // }}}
    // {{{ is_zone_user()

    /**
      * Test if user is zone user or up
      *
      * @param  int     zone ID to check against
      * @return boolean true if user has zone user rights or above
      */
	function is_zone_user($zone = false) {

        return ($this->get_access_level($zone) >= ZONE_USER);

	} //is_zone_user

    // }}}
    // {{{ is_guest()

    /**
      * Test if user has guest rights or above
      *
      * @param  int     zone ID to check against
      * @return boolean true if user has guest rights or above
      */
	function is_guest($zone = false) {

        return ($this->get_access_level($zone) >= GUEST);

	} //is_guest

    // }}}

    /* Internal Function */
    // {{{ _update_password()

    /**
      * This function is designed to be called at the start of
      * inserts and updates, to hash the password if necessary
      *
      * The internal variable 'password' WILL be changed if needed
      */
    function _update_password()
    {
        // we only need to do this for mysql users, so:
        if ($this->type != 'auth_maintain') {
            return;
        }


        $dbdo = DB_DataObject::Factory('user');
        $dbdo->username = $this->username;
        $dbdo->type     = 'auth_maintain';

        // if they exist, we need to connect to the database,
        // compare the current password to the one in the DB,
        // and see if it's already hashed
        if ($dbdo->find(true)) {

            // if the password has not changed, leave it alone!
            if (!strcmp($dbdo->password, $this->password)) {
                return;
            }
        }

        // the password has changed, we need to sha-1 hash it
        $this->password = sha1($this->password);

    }

    // }}}

    /* Maintain Overloads */
    // {{{ _history_text()
    function _history_text()
    {
        // : USERNAME (ldap/mysql/etc)
        return "$this->username ($this->type)";
    }
    // }}}

}

?>
