<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC2/class/htdocs/DB_DataObjectDataListSource.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Authors: Daniel Robert <robert@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: DB_DataObjectDataListSource.php 3926 2005-08-27 00:19:41Z robert $

/**
  * DB_DataObjectDataListSource.php - contains the 
  * {@link DB_DataObjectDataListSource} class
  *
  * @author $Author: robert $
  * @filesource
  * @package    htdocs
  * @subpackage datalists
  */

require_once("$phphtmllib/widgets/data_list/SQLDataListSource.inc");

/**
  * This class provides DB abstraction for phpHtmlLib datalists
  * using DB_DataObjects as the interface
  *
  * @link   http://phphtmllib.newsblob.com/javadoc.php phpHtmlLib API Documentation
  * @link   http://phphtmllib.newsblob.com/doc/phpHtmlLib/SQLDataListSource.html 
            SQLDataListSource API Documentation 
  * @package    htdocs
  * @subpackage datalists
  */
class DB_DataObjectDataListSource extends SQLDataListSource
{
    /** The DB_DataObject used for this source */
    var $_dbdo;

    /** List of variables to return in get_next_data_row() */
    var $_vars;
    var $count = 0;

    // {{{ constructor

    /**
      * Constructor
      *
      * @param  DB_DataObject
      */
    function DB_DataObjectDataListSource( &$db )
    {
        $this->_dbdo = $db;
        $this->_vars = array();
    }

    // }}}

    /* Overloads from DataListSource */

    // {{{ add_column()
    function add_column($title, $data_name, $sortable=FALSE,
            $searchable=FALSE, $sortorder="ASC")
    {
        $this->_vars[] = $data_name;

        parent::add_column($title, $data_name, $sortable=FALSE,
                $searchable=FALSE, $sortorder="ASC");
    }
    // }}}
    // {{{ do_query()

    /* Required Overloads */
    function do_query()
    {
        //DB_DataObject::debugLevel(3);
        return $this->_dbdo->find();
    }

    // }}}
    // {{{ get_next_data_row()

    function get_next_data_row()
    {
        if ($this->_dbdo->fetch())
            return $this->_dbdo->toArray();
        else
            return array();
    }

    // }}}

    /* Overloads from SQLDataSource */
   
    // {{{ setup_db_options()
    
    /**
      * Although technically overloading setup_db_functions from
      * SQLDataListSource, we're using this function to set DB_DataObject
      * values using the native ->setFrom() method
      *
      * @param  array   array of values
      */
    function setup_db_options($opts)
    {
        $this->_dbdo->setFrom($opts);
    }

    // }}}
    // {{{ build_query()

    // only overloaded so it doesn't try and generate a raw query
    function build_query($limit_flag=TRUE)
    {
        $this->build_where_clause();
        $this->build_order_clause();
        $this->build_groupby_clause();

        // now, get the total rows before we add the limit
        $this->set_total_rows($this->count());

        if ($limit_flag)
            $this->build_limit_clause($this->get_offset(), $this->get_limit());

        // for compatibility
        return 'query';
    }
    // }}}
    // {{{ build_where_clause()

    function build_where_clause()
    {
        // build the where clause
        $where_clause = $this->_db_options['where_clause'];
        if ($where_clause)
            $this->_dbdo->whereAdd($where_clause);

        // build the search clause
        $search_clause = $this->build_search_clause();
        if ($search_clause) {
            $this->_dbdo->whereAdd($search_clause);
        }

    }

    // }}}
    // {{{ build_order_clause()

    function build_order_clause()
    {
        // clear the previous order
        $this->_dbdo->orderBy();

        // set up the order
        $order_clause = parent::build_order_clause();

        if ($order_clause)
            $this->_dbdo->orderBy($order_clause);
    }

    // }}}
    // {{{ simple_search_where_clause()

    function simple_search_where_clause()
    {
        // make sure we're searching for something
        if (!$this->get_searchby() || !$this->get_searchby_value()) {
            return '';
        }

        $mod = $this->get_simplesearch_modifier();
        $mod = ($mod) ? $mod : SEARCH_EXACT;

        // if we're doing anything tricky, like using 'LIKE' or a comparison,
        // just use a whereAdd
        if ($mod != SEARCH_EXACT) {
            return parent::simple_search_where_clause();

        } 

        /* otherwise, rely on the db_dataobject's internal setters
         * and have it auto-formulate the where clause(s) */

        // ucfirst because the function is setColumn_name
        $func = 'set' . ucFirst($this->get_searchby());

        // set the value using the object's setter
        $this->_dbdo->$func($this->get_searchby_value());

        return '';

    }

    // }}}
    // {{{ setup_order()

    // helper function for build_order_clause
    function setup_order($orderby, $reverseorder, $icase_sort=FALSE)
    {
        if ($icase_sort)
            $orderby_clause = "lower($orderby)";
        else
            $orderby_clause = $orderby;

        $orderby_clause .= ($reverseorder == 'true') ? ' DESC' : ' ASC';

        $secondary = $this->get_secondary_orderby();
        
        if (count($secondary) > 0)
            $oderby_clause .= ','.implode(',', $secondary);

        return $orderby_clause;
    }

    // }}}
    // {{{ build_groupby_clause()

    function build_groupby_clause()
    {
        // clear the previous groupby
        $this->_dbdo->groupBy();

        if ($this->_db_options['groupby_clause'])
            $this->_dbdo->groupBy($this->_db_options['groupby_clause']);

    }

    // }}}
    // {{{ build_limit_clause()

    function build_limit_clause($offset, $limit)
    {
        // clear the previous
        $this->_dbdo->limit();

        if (!$offset)
            $offset = 0;

        $limit_clause = $offset;

        if ($limit)
            $this->_dbdo->limit($offset, $limit);
        else
            $this->_dbdo->limit($offset);

    }

    // }}}
    // {{{ count()

    function count()
    {
        // make a copy of the DBDO, but reset the limits
        // clone works because it is defined in DB_DataObject
        $copy = clone($this->_dbdo);
        //$copy->limit();
        $count = $copy->find();
        unset($copy);
        return $count;
    }

    // }}}

   /* Getters */

//{{{   get_dbdo()

    /**
     *  Returns the DBDO object
     *
     *  @return DB_DataObject   $_dbdo  The stored DB_Dataobject
     */
    function get_dbdo()
    {
        return $this->_dbdo;
    }

//}}} 

}
