<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// |            Zack Bartel <bartel@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: CnameEditForm.php 4606 2006-07-12 22:41:37Z morgabra $


require_once(FORMDIR."/CnameCreateForm.php");

class CnameEditForm extends CnameCreateForm
{

//{{{   form_init_elements()i

    /**
     *  This method initializes all of the form elements that are going to be
     *  used.
     */
    function form_init_elements()
    {

        $name = new FEText(_("Name"), false);
        $check  = new FECheckbox(_("Entire Domain"));
        $domains = new FEListBox(_("Domain"),false);
        $server = new FEText(_("Server Name"),true);
        $ttl = new FEText(_("TTL"),true);
        $comment = new FETextArea(_("Comment"), true, 5, 40);

        //populate the listbox
        $domains->set_list_data($this->_domains());
        $this->add_elements($name, $check,$domains,$server,$ttl, $comment);

    } 

//}}}
//{{{ i  form_init_data()

    /**
     *  This method populates the fields with the initial values in the database.
     */
    function form_init_data()
    {
        global $conf;
        $cname = new Cname($_REQUEST['id']);
        $cname->query("BEGIN");
        $initvals = $cname->toArray();
        $formvals = $this->remap($initvals, FORM);

        foreach($formvals as $label=>$val) {
            $this->set_element_value($label, $val);
        }

        $this->set_element_value(_('TTL'), $conf['soa']['default_cname_ttl']);
        $this->set_element_value(_('Comment'),_("Modified CNAME"));
        $cname->query("COMMIT");
    }

//}}}
//{{{   form_action()

    /**
     *  This method is what actually updates the CNAME when if the field values
     *  are all valid.
     *
     *  @return Boolean     True if action was successful
     */
    function form_action() 
    {
        global $user;
        
        $res = true;
        $vals = $this->get_all_element_values();

        switch(strtolower($this->get_action())) {

        case 'submit':
            $cnm = new Cname();
            $cnm->query("BEGIN");
            $cnm->setAll();
            $cnm->setFrom($this->remap($vals,DBDO));
            $cnm->zone = $_SESSION['zone'];
            $cnm->id = $_REQUEST['id']; //explicitly set the ID

            if($vals[_('Entire_Domain')]) {

                $dom = new Domain($vals[_("Domain")]);
                
                //Make sure it is indeed a subdomain
                if(!$dom->master_domain) {
                    $res = false;
                    $this->add_error(_("Domain"),
                        _("Domain's master domain is not in Maintain."));
                }
                //the domain is the master
                $cnm->setDomain($dom->master_domain);
                //the name is the subdomain
                $cnm->setName(strtok($dom->name,'.'));
                
            }
            if($cnm->update() === false) {
                $cnm->query("ROLLBACK");
                $this->add_error(_("ERROR"), _("Update query failed"));
                $res = false;
            } else {
                $cnm->query("COMMIT");
                $cnm->write_history($vals[_("Comment")], ACTION_MODIFY, $user->username, $_SESSION['zone']);
            }
            break;
 
        default:
            die_hard(_("Invalid form action"));
            break;
        }

        return $res;
    }

//}}}
//{{{   form_hidden()

    /**
     *  Add some hidden values that we'll need later.  The ID of the CNAME and the
     *  get action so we can get back to the form.
     *
     *  @return Container   Container that has the hidden elements in it
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));
    }

//}}}i
//{{{   form_success()i

    /**
     *  This method displays a success message if the action worked
     *
     *  @return DIVTag  $div    Div with the success message in it
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $div->add(_("CNAME successfully updated!"),html_br());
        $indlink = html_a("index.php", "[Return to Home]");
        $link = html_a("cname.php", "[Return to CNAME List]");
        $div->add(container($indlink, $link));

        return $div;
    }

//}}}

}

?>
