<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: WorkgroupEditForm.php 4606 2006-07-12 22:41:37Z morgabra $

/**
 *  WorkgroupEditForm.php - This file contains the {@link WorkgroupEditForm}
 *                          class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  WorkgroupEditForm is the form used to edit workgroups.  
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class WorkgroupEditForm extends MaintainForm
{

//{{{   vars

    /**
     *  Used to convert between form labels and DBDO vars
     */
    var $map = array('Name' => 'name');
    
//}}}

//{{{   form_init_elements()

    /**
     *  Initialize the element(s) that are going to be used in the form.
     *  Workgroup happens to be freakishly simple...
     */
    function form_init_elements()
    {
        $name = new FEText(_("Name"),true);

        $this->add_element($name);
    }

//}}}
//{{{   form_init_data()

    /**
     *  Gather the initial values and shove them in the form as the defaults.
     *  Though there is only one element, I still treat this as if there were more,
     *  this is to make it more extensible in the long run.
     */
    function form_init_data()
    {
        $wg = new Workgroup($_REQUEST['id']);
        $formary = $this->remap($wg->toArray(), FORM);
        foreach($formary as $label=>$val)
            $this->set_element_value($label, $val);
    }

//}}}
//{{{   form_action()

    /**
     *  This method processes their button press and takes care of the whole
     *  talking to the database to make stuff happen thing.
     *
     *  @return Boolean True if the action was successful
     */
    function form_action()
    {
        global $user;

        $res = true;
    
        switch(strtolower($this->get_action())) {
        
        case 'submit':
            $vals = $this->get_all_element_values();
            
            $wg = new Workgroup($_REQUEST['id']);
            $wg->query("BEGIN");
            $wg->setFrom($this->remap($vals, DBDO));

            if($wg->update() === false) {
                $wg->query("ROLLBACK");
                $this->add_error(_("ERROR"),_("Could not update DB Entry"));
                $res = false;
            } else {
                $wg->query("COMMIT");
                $wg->write_history(_("Workgroup Updated"), ACTION_MODIFY,
                                    $user->username);
            }
            break;
        default:
            $this->add_error(_("ERROR"), _("Unknown form action"));
            break;
            
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  If the action WAS successful tell them so.
     *
     *  @return DIVTag  $div    Div with a success message inside.
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $div->add("Workgroup Successfully Updated!");
        return $div;
    }

//}}}
//{{{   form_hidden()

    /**
     *  Add some hidden goodness in the form, these are items that need to be
     *  carried over throughout the form, otherwise we'll never get back to it
     *  to actually perform the proper action.
     *
     *  @return Container   Container with hidden elements.
     */
    function form_hidden()
    {
        $objid = new FEHidden("id",$_REQUEST['id']);
        $action = new FEHidden("action", "edit");

        $this->add_elements($objid, $action);

        return container($this->element_form("id"), $this->element_form("action"));
    }

//}}}

}

?>
