<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/tags/release-3.0.0-RC2/htdocs/host.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Authors: Michael Clay <claym@osuosl.org>                               |
// |          Zack Bartel <bartel@osuosl.org>                               |
// +------------------------------------------------------------------------+ 
// $Id: host.php 4758 2006-08-18 21:37:02Z schonstal $

/**
  * Host pages
  *
  * @package    htdocs
  */

require_once("includes.php");
require_once(HTCLASS."/datalist/HostDataList.php");
require_once(FORMDIR."/HostDisplayForm.php");
require_once(FORMDIR."/HostBulkDeleteConfirm.php");
require_once(FORMDIR."/HostCreateForm.php");
require_once(FORMDIR."/HostBulkUpdateForm.php");
require_once(FORMDIR."/HostBulkZoneMoveForm.php");
require_once(FORMDIR."/HostBatchMoveForm.php");
require_once(FORMDIR."/HostBatchCopyForm.php");
session_start();

setup_user_environment();

$title = html_span("title");

$skin = Skin::Factory($user->color_template); 

$content = container();

$res = true;  //Result for errors

switch(strtolower($_REQUEST['action'])) {

case 'edit':
    $title->add(_("Modifying Host"));
    $content->add($title);
    
     if(!$user->is_zone_user($zone) || !isset($_REQUEST['id']))
        header("Location: denied.php");
        
    $host = new Host($_REQUEST['id']);
    $zoneobj = new Zone($_SESSION['zone']);
    $nextip = $zoneobj->get_next_host_by_ip($host);
    $nextname = $zoneobj->get_next_host_by_name($host);
   
    /* Action Menu*/
    $am =& $skin->getActionMenu();
    $edit_link = html_a($_SERVER["SCRIPT_NAME"]."?action=edit&amp;id=".$_REQUEST['id'],
                        _("Edit"));
    $del_link = html_a($_SERVER["SCRIPT_NAME"]."?action=delconfirm&amp;id=".$_REQUEST['id'],
                        _("Delete"));
    $create_link = html_a($_SERVER["SCRIPT_NAME"]."?action=create",_("Create New Host"));
    $list_link = html_a($_SERVER["SCRIPT_NAME"], _("Show Host List"));
    $nextname_link = html_a($_SERVER["SCRIPT_NAME"].'?action=edit&amp;id='.$nextname,
                            _("Edit Next By Name"));
    $nextip_link = html_a($_SERVER["SCRIPT_NAME"]."?action=edit&amp;id=".$nextip,
                            _("Edit Next By IP"));

    $option_link = html_a("option.php?action=show&amp;object_type=host&amp;object_id={$_REQUEST['id']}", _("View DHCP Options for this Host"));
    
    $am->add($edit_link, $del_link);

    //Since the functions may return false we need to make sure there IS a next host
    //Before adding supposed links to them.
    if($nextname)
        $am->add($nextname_link);
    if($nextip)
        $am->add($nextip_link);

    $am->add($create_link, $list_link, $option_link);

        
    $viewlinks = container(html_a("host.php?action=edit&amp;id=".$_REQUEST['id'].
                            "&amp;view=advanced", _("[Advanced]")),
                            html_a("host.php?action=edit&amp;id=".$_REQUEST['id'].
                            "&amp;view=basic", _("[Basic]")));
    $content->add(html_br(2),$am,html_br(2));
    if(!isset($_REQUEST["hostedit_form_visited"]) && 
        !isset($_REQUEST['movehost_form_visited']) &&
        !isset($_REQUEST['zonechange_form_visited']))
        $content->add($viewlinks);

    $hdf = new FormProcessor(new HostDisplayForm(),"hostedit");
    $hdf->set_render_form_after_success(false);
    
    $content->add(html_br(), show_live_time($_REQUEST['id'], _("Host")));
    $content->add(html_br(), $hdf);

    //If the user is an admin
    if(!$user->is_admin())
        break;
   
    //Don't display the other forms if we're processing a different one.
    if(!isset($_REQUEST["hostedit_form_visited"]) || $hdf->has_errors()) {
    
        //If we've already tried to move the zone OR we submitted
        //our new IP, workgroup, etc we stick with this form for processing.
        if($_REQUEST['movehost_form_visited'] == 1 ||
            ($_REQUEST['zonechange_form_visited'] == 1 &&
            strcasecmp($_REQUEST['_form_action'],"cancel"))) {
           
            $zcf = new FormProcessor(new ZoneChangeForm(),"zonechange");
            $content->add($zcf);
            
        }
        else { //if we haven't tried to move zones or we canceled.
            $mhf = new FormProcessor(new MoveHostForm(),"movehost");    
            $content->add($mhf);
        }
 
    }
    break;
    
case _('bulk host delete'):

    $title->add(_("Bulk Host Delete"));
    $content->add($title);
    
    if(!$user->is_zone_user($zone))
        header("Location: denied.php");
        
    if(empty($_POST['hostlist']))
        header("Location: host.php");
        
    $bdcf = new FormProcessor(new HostBulkDeleteConfirm(), "delconfirm");
    $bdcf->set_render_form_after_success(false);
    $content->add($bdcf);
    break;

case _('host batch move'):

    $title->add(_("Host Batch Move"));

    if(!$user->is_zone_user())
        header("Location: denied.php");

    if(empty($_POST['hostlist']))
        header("Location: host.php");
        
    $bhm = new FormProcessor(new HostBatchMoveForm(), "batchmove");
    $bhm->set_render_form_after_success(false);
    $content->add($title, $bhm);
    break;

case _('host batch copy'):

    $title->add(_("Host Batch Copy"));

    if(!$user->is_zone_user())
        header("Location: denied.php");
    
    if(empty($_POST['hostlist']))
        header("Location: host.php");

    $bhc = new FormProcessor(new HostBatchCopyForm(), "batchcopy");
    $bhc->set_render_form_after_success(false);
    $content->add($title, $bhc);
    break;
       

case _('bulk host zone move'):

    $title->add(_("Bulk Host Zone Move"));

    if(!$user->is_zone_user())
        header("Location: denied.php");

    if(empty($_POST['hostlist']))
        header("Location: host.php");
        
    $zbhm = new FormProcessor(new HostBulkZoneMoveForm(), "hosts");
    $zbhm->set_render_form_after_success(false);
    $content->add($title, $zbhm);
    break;

case _('bulk host update'):
    if (!$user->is_zone_user())
        header("Location: denied.php");
    if(!isset($_POST['hostlist']))
        header("Location: host.php");
    $title->add(_("Select fields to update"));    
    $hbuf = new FormProcessor(new HostBulkUpdateForm(), "hosts");
    $hbuf->set_render_form_after_success(false);
    $content->add($title, $hbuf);
    break; 

case 'update':
    $title->add(_("Bulk Host Update")); 
    $content->add($title);

    if (!$user->is_zone_user())
        header("Location: denied.php");
    
    $hl = new HostDataList("Hosts","100%","name");
    $hl->user_setup();
    $content->add($hl);

    break;
case 'delconfirm':

    $title->add(_("Deleting Host"));
    $content->add($title);
    
    if(!$user->is_zone_user($zone))
        header("Location: denied.php");
        
    $dcf = new FormProcessor(new HostDeleteConfirm(), "delconfirm");
    $dcf->set_render_form_after_success(false);
    $content->add($dcf);
    break;

case 'create':
    $title->add(_("Creating Host"));
    $content->add($title);
    $am =& $skin->getActionMenu(); 
    $showall_link = html_a($_SERVER["SCRIPT_NAME"]."?action=show",_("Show Host List"));
    $am->add($showall_link);
    $content->add(html_br(2), $am, html_br(2));

    /* The advanced/basic links */
    $viewlinks = container(html_a("host.php?action=create&amp;view=advanced", _("[Advanced]")),
                            html_a("host.php?action=create&amp;view=basic", _("[Basic]")));

    /* Check admin status */
    if (!$user->is_zone_user($_SESSION['zone']))
        header("Location: denied.php");
    $hcf = new FormProcessor(new HostCreateForm(), "create");
    $hcf->set_render_form_after_success(false);
    if(!$hcf->is_action_successfull())
        $content->add($viewlinks);
    $content->add($hcf);
    
    break;

case 'createconfirm':
    
    header("Location: host.php?action=show");
    break;

case 'enable':
case 'disable':
    if (!$user->is_zone_user())
        header("Location: denied.php");
    if (!isset($_REQUEST['id']))    {
        var_dump("No host id");
        break;
    }
        
    $host = new Host($_REQUEST['id']);
    $host->setEnabled((strcasecmp($_REQUEST['action'], 'enable') ? 0 : 1));
    $res = $host->validateEnabled();
    if($res === true)
        $host->update();
    unset($_REQUEST['id']);
   
case 'show':

default:

    $title->add(_("Showing Hosts"));
    $content->add($title);
    
    /* If you aren't a zone user then gtfo */
    if(!$user->is_zone_user($zone))
        header("Location: denied.php");
    
    /* Datalist or display object */
    if(isset($_REQUEST['id'])) {
        $host = new Host($_REQUEST['id']);
        $zoneobj = new Zone($_SESSION['zone']);
        $nextip = $zoneobj->get_next_host_by_ip($host);
        $nextname = $zoneobj->get_next_host_by_name($host);
       
        /* Action Menu*/
        $am =& $skin->getActionMenu();
        $edit_link = html_a($_SERVER["SCRIPT_NAME"]."?action=edit&amp;id=".$_REQUEST['id'],
                            _("Edit"));
        $del_link = html_a($_SERVER["SCRIPT_NAME"]."?action=delconfirm&amp;id=".$_REQUEST['id'],
                            _("Delete"));
        $create_link = html_a($_SERVER["SCRIPT_NAME"]."?action=create&amp;view",_("Create New Host"));
        $list_link = html_a($_SERVER["SCRIPT_NAME"], _("Show Host List"));
        $nextname_link = html_a($_SERVER["SCRIPT_NAME"].'?action=edit&amp;id='.$nextname,
                                _("Edit Next By Name"));
        $nextip_link = html_a($_SERVER["SCRIPT_NAME"]."?action=edit&amp;id=".$nextip,
                                _("Edit Next By IP"));

        $am->add($edit_link, $del_link);

        //Since the functions may return false we need to make sure there IS a next host
        //Before adding supposed links to them.
        if($nextname)
            $am->add($nextname_link);
        if($nextip)
            $am->add($nextip_link);

        $am->add($create_link, $list_link, $option_link);
        
        $content->add(html_br(2),$am, html_br(2));
        
        $content = Module::apply_filter("object_info",$content,$host);
    } else {        
        /* Action Menu */
        $am =& $skin->getActionMenu();
        $am->add(html_a($_SERVER["SCRIPT_NAME"]."?action=create",_("Create a new Host")));
        $content->add(html_br(2),$am,html_br(2));
        if($res !== true) {
            $error = html_span('error');
            $error->add($res);
            $content->add($error,html_br(2));
        }
        $hl = new HostDataList("Hosts","100%","name");
        $hl->user_setup();
        $content->add($hl);
    }
    break;
    
}

print $skin->display($content);
?>
